lexer grammar WikiLexer;
options {
    language = 'CSharp2';
}
@namespace { ProjectBase.Tools.Wiki
}
@lexer::members  {
    int inliner = 0;
    bool insideValue  = false; 
}


/* BLOCKS */
CODE_BEGIN :'[CODE';   CODE_END  :'CODE]';  
HTML_BEGIN :'[HTML';   HTML_END  :'HTML]';  
XML_BEGIN  :'[XML';    XML_END   :'XML]';  

PRE_BEGIN  :'[PRE';    PRE_END   :'PRE]';  
BQ_BEGIN   :'[BQ';     BQ_END    :'BQ]';  

LI_BEGIN   :'[LI';     LI_END    :'LI]';  
NUM_BEGIN  :'[NUM';    NUM_END   :'NUM]';



/* LEXER */
NEWLINE :   ('\r'? '\n') ;
WS      : '  ' | ' ' | '\t';

COMMENT : '/*'   ( options {greedy=false;} : . )* '*/'
        | '&lt;!--' ( options {greedy=false;} : . )* '--&gt;'
        | '/// '  ( options {greedy=false;} : . )* NEWLINE
        | '// '  ( options {greedy=false;} : . )* NEWLINE
        | '<!--' ( options {greedy=false;} : . )* '-->'
;
QUOTED  :  '"' ( options {greedy=false;} : . )* '"'
        | '&quot;' ( options {greedy=false;} : . )* '&quot;'
;
       
LT      :  ('&lt;' | LTSIGN) SLASH? ;
GT      :  SLASH? ( '&gt;' | GTSIGN) ;

ESCAPE  : ('[[]' | '[]]' | '[:]' | '[|]' | '[<]' | '[>]' | '[&gt;]' | '[&lt;]' | '["]' | '[&quot;]');

A       : ('[URL:' ) { inliner++; insideValue = true; };
NAME    : ('[NAME:') { inliner++; insideValue = true; };
IMG     : ('[IMG:' ) { inliner++; insideValue = true; };

//OPEN  : '[' ; // '\u005b' - defined elsewhere
CLOSE   : {inliner > 0}?=> ']'               { inliner--; };
VLINE   : {inliner > 0}?=> WS? '|' WS?        { insideValue = false; };
COLON   : {inliner > 0 && !insideValue}?=> WS? ':' WS? { insideValue = true; }; 
NOCONTROL: {inliner==0}?=> ( ']' | '|' ) 
         | {inliner==0||insideValue}?=> ( 'http://www.' | 'http://' | ':') 
         | {inliner==0||insideValue}?=> ( 'HEIGHT' | 'TEXT' | 'WIDTH' | 'FLOAT' )
;
  
/* ATTRIBUTES */
ATTR_TEXT    : {inliner > 0 && !insideValue}?=> ('TEXT');
ATTR_HEIGHT  : {inliner > 0 && !insideValue}?=> ('HEIGHT');
ATTR_WIDTH   : {inliner > 0 && !insideValue}?=> ('WIDTH');
ATTR_FLOAT   : {inliner > 0 && !insideValue}?=> ('FLOAT');

INLINE_BEGIN: '[' (STYLES | COLORS) ':' { inliner++; };
 
//SEMI  : ';'; // '\u003b' - defined elswhere
COLORS  : ( 'BLUE' | 'RED' | 'GREEN' | 'OLIVE' | 'LIME' | 'PURPLE' 
        | 'YELLOW' | 'MAROON' | 'SILVER' | 'ORANGE' | 'NAVY' | 'TEAL' | 'PINK' )
;
STYLES: ( 'B'  | 'I' | 'U' | 'S' | 'SAMP' | 'V'   | 'VAR' 
        | 'SUB' | 'SUP' | 'BIG' | 'SMALL' | 'DFN' | 'DEF')
;


/* FRAGMENTS for easier reading */
fragment EXCLAM: '!'; // '\u0021'
//--fragment QSIGN : '"'; // '\u0022' - defined elsewhere
fragment HASH  : '#'; // '\u0023'
fragment AMPR  : '&'; // '\u0026'
fragment LEFT  : '('; // '\u0028'
fragment RIGHT : ')'; // '\u0029'
fragment DOT   : '.'; // '\u002e'         
fragment SLASH : '/'; // '\u002f'
//fragment COLON : ':'; // '\u003a'  - defined elsewhere
fragment SEMI  : ';'; // '\u003b'
fragment LTSIGN: '<'; // '\u003c' 
fragment EQUAL : '='; // '\u003d'
fragment GTSIGN: '>'; // '\u003e' 
fragment QUEST : '?'; // '\u003f'
//fragment OPEN  : '['; // '\u005b'  - defined elsewhere
//fragment CLOSE : ']'; // '\u005d'  - defined elsewhere
fragment LCB   : '{'; // '\u007b'
//fragment VLINE : '|'; // '\u007b'  - defined elsewhere
fragment RCB   : '}'; // '\u007d'
fragment DIGIT   : '\u0030'.. '\u0039';
fragment UPPER  : '\u0041'.. '\u005a';
fragment LOWER  : '\u0061'.. '\u007a';
fragment NOLEADING : '\u0024'| '\u0025' | '\u0027' | '\u002a'..'\u002d' | '\u0040' | '\u005c' | '\u005e' .. '\u0060'
;
fragment UNICODREST : '\u007e' .. '\ufffe'; // the last is the EOF
fragment LETTER: DIGIT | LOWER | UPPER | HASH | SEMI | SLASH | NOLEADING | UNICODREST
;


/* SPECIAL groups definition */
ENCODED : AMPR ((LOWER | UPPER)+ | (HASH DIGIT+) ) SEMI;
SYMBOLS:  EXCLAM | EQUAL | QUEST | LEFT | RIGHT | DOT | LCB | RCB
;
CSHARP_BASE    : ( 'class' | 'public' | 'protected' | 'private' | 'abstract' 
                  | 'virtual' | 'new' | 'struct' | 'const'
                  | 'void' | 'interface' | 'get' | 'set'
                  | 'readonly' | 'override' | 'default'| 'enum' | 'sealed ' 
                  | 'namespace' | 'params' | 'ref' | 'out' | 'base' 
                  | 'unchecked' | 'extern' | 'unsafe'| 'checked' | 'operator' | 'implicit' | 'explicit' 
                  );
CSHARP_TYPES    : ( 'string' | 'int' | 'long' | 'short' | 'byte' 
                  | 'decimal' | 'sbyte' | 'uint' | 'ulong' | 'ushort' 
                  | 'char' | 'object'  | 'Guid' | 'var' //| 'float'
                  | 'function'
                  );
CSHARP_FLOW     : ( 'switch' | 'case' | 'if' | 'else' | 'while' | 'do' | 'for' 
                  | 'foreach' | 'break' | 'continue' | 'return' | 'using'
                  | 'try' | 'catch' | 'goto' | 'throw' | 'finally' | 'in' | 'lock' 
                  );
CSHARP_SPECIAL  : ( 'as' | 'is' | 'null' | 'true' | 'false' | 'event' | 'this'
									| 'Equals' | 'Is()' | 'IsNull()' | 'IList' 
                  | 'params' | 'typeof' | 'delegate' 
                  | 'Where' | 'Select' | 'Any' | 'IDictionary' | 'ToDisplay' | 'Func'
                  );
HTML_BASE       : ( 'href' | 'style' | 'rel' | 'title' | 'src' | 'value' 
                  | 'id' | 'name' | 'type' | 'link' 
                );
IBTOKEN     : '|CITE ' | '|BQ ' 
            | '|P '  | '|PR ' | '|PC ' 
            | '|P1 ' | '|P2 ' | '|P3 '        
            | '|H1 ' | '|H2 ' | '|H3 ' | '|H4 ' | '|H5 ' | '|H6 '
;
WORD: LETTER+
;
