parser grammar WikiParser;
options {
    language = 'CSharp2'; output=AST; tokenVocab=WikiLexer; k=12;
}
tokens{
  CODE_BEGIN; CODE_END  ;  
  HTML_BEGIN;  HTML_END  ;  
  XML_BEGIN ;  XML_END   ;  

  PRE_BEGIN  ;  PRE_END    ;
  BQ_BEGIN  ;  BQ_END    ;

  LI_BEGIN  ;  LI_END    ;  
  NUM_BEGIN  ;  NUM_END    ;
  /* imaginary */
  ELEMENT;
  NAME_ELEMENT;
  A_ELEMENT;  
  IMG_ELEMENT;
  
  CODE;
  XML;
  HTML;
  INLINE;
  TAG;
  LIST;
  NUMERIC;
}
@namespace { ProjectBase.Tools.Wiki
}

/* starting point */
execute: statement* EOF!
;

statement     : (blocks | inlineBlocks )
;
/* blocks - the parents only; could be child */
blocks        : (code | pre | bq | li | num | html | xml)
;
/* inlines - the child only */
inlineBlocks  : (inlineBlock | emptyBlock )
;

/* inner elements of the blocks and inlines */
elements      : ( blocks | inlineBlocks )+
;
inlineElements: ( inline | atom | text )+
;

/* block */
code  :    CODE_BEGIN   elements CODE_END -> ^(CODE elements); 
html  :    HTML_BEGIN   elements HTML_END -> ^(HTML elements); 
xml   :    XML_BEGIN   elements XML_END   -> ^(XML  elements); 

bq    :    BQ_BEGIN     elements BQ_END   -> ^(ELEMENT["bq"] elements);
pre   :    PRE_BEGIN   elements PRE_END   -> ^(ELEMENT["pre"] elements);

li    :    LI_BEGIN     elements LI_END   -> ^(LIST elements); 
num   :    NUM_BEGIN   elements NUM_END   -> ^(NUMERIC elements); 

/* inline blocks - terminated with the newline */
inlineBlock: ib=IBTOKEN inlineElements ( terminator | EOF )?  
                -> ^(ELEMENT[$ib] inlineElements)
           | terminator
;
emptyBlock : inlineElements ( terminator | EOF )?
                -> ^(ELEMENT["empty"] inlineElements)
;

/* inlines - specific pieces defined without terminator!!! */
inline      : ib=INLINE_BEGIN inlineElements terminator? CLOSE 
                -> ^(INLINE[$ib] inlineElements)
;

atom  : (anchor | name | image | tag | LT | GT)
;

/* extern */
anchor:  A href=text WS* CLOSE 
          -> ^(A_ELEMENT $href ^(ATTR_TEXT $href ) ) 
      | A text VLINE txt=attrText WS* CLOSE 
          -> ^(A_ELEMENT text ^(ATTR_TEXT $txt) )
;
name  :  NAME text CLOSE                       -> ^(NAME_ELEMENT text)
;
image :   IMG text imageAttrs WS* CLOSE        -> ^(IMG_ELEMENT text imageAttrs)
;

tag   :   lt=LT  text gt=GT                   -> ^(TAG $lt text $gt)
;

/* extern attr */
attrText  : ATTR_TEXT COLON textAndImage      -> textAndImage
;
textAndImage: (text | image)+
;

imageAttrs: (widthText | heightText | floatText)+
          |   -> ^(ATTR_FLOAT)
;

heightText : VLINE ATTR_HEIGHT COLON WS* t=WORD WS*       -> ^(ATTR_HEIGHT $t);
widthText  : VLINE ATTR_WIDTH  COLON WS* t=WORD WS*       -> ^(ATTR_WIDTH $t) ;
floatText  : VLINE ATTR_FLOAT  COLON WS* t=WORD WS*       -> ^(ATTR_FLOAT $t) ;

/* basic */
text: (WS | COMMENT | QUOTED | WORD | ESCAPE | SYMBOLS | NOCONTROL | ENCODED 
     | COLORS | STYLES | CSHARP_BASE | CSHARP_FLOW | CSHARP_SPECIAL | CSHARP_TYPES | HTML_BASE)+
;

terminator: NEWLINE+ 
;
