tree grammar WikiTree;

options {
    language = 'CSharp2'; output=AST; ASTLabelType=CommonTree; tokenVocab=WikiParser;
}
@header{
using System.Text;
using System.Collections.Generic;
}
@namespace { ProjectBase.Tools.Wiki
}

/*
These declaration are intended to be as easy as possible.
Some complicated AST to HTML conversions are moved to the 
partial class - of this file (resulted code of the ANTLR generator)

see: WikiTree.Code.cs
for more
*/

execute  : statement*;

statement: block | lists | NEWLINE ->
;
block    :   ^(b=ELEMENT { BlockTag($b.Text, false); } (element)*  { BlockTag($b.Text, true); } )
         |   ^(ce=CODE   { BlockTag($ce.Text, false); } (element)*  { BlockTag($ce.Text, true); }  )
         |   ^(ht=HTML   { BlockTag($ht.Text, false); } (element)*  { BlockTag($ht.Text, true); }  )
         |   ^(xm=XML    { BlockTag($xm.Text, false); } (element)*  { BlockTag($xm.Text, true); }  )
;
lists    :   ^(l=LIST    { BlockTag("UL", false); } NEWLINE* (listItem)* NEWLINE* { BlockTag("UL", true); } )
         |   ^(n=NUMERIC { BlockTag("OL", false); } NEWLINE* (listItem)* NEWLINE* { BlockTag("OL", true); } )
;
listItem :   ( { BlockTag("LI", false); }
                element  { BlockTag("LI", true); }
             )
;


element  : block | lists | atom | codeText | tag
;
inlineElement: atom | codeText | tag
;
atom    : inline | anchor | name | image 
;


inline  :   ^(i=INLINE    { InlineTag($i.Text, false); } 
                (inlineElement)*   { InlineTag($i.Text, true); } 
            )
;        
anchor  : ^(A_ELEMENT   { SpecialTextStart(); } (pureText)* { SpecialTextEnd(AnchorTemplate);}
                        attrText { Append("</a>");}
            )
;
name     : ^(NAME_ELEMENT{ Append("<a name=\""); } (pureText)* { Append("\" ></a>");} 
            )
;
image    : ^(IMG_ELEMENT { SpecialTextStart(); } (pureText)* { SpecialTextEnd(ImageTemplate);}
                        (heightText | widthText | floatText)*  { Append("\" />");}
            )
;

tag      : ^(TAG { TagElement(false); } (codeText)*) { TagElement(true); } 
;

attrText : ^(ATTR_TEXT (simpleText | image)* )
;
heightText: ^(ATTR_HEIGHT pt=WORD { ImageAttr($pt.Text, "height"); } )
;
widthText : ^(ATTR_WIDTH   pt=WORD { ImageAttr($pt.Text, "width"); } )
;
floatText : ^(ATTR_FLOAT   pt=WORD { ImageAttr($pt.Text, "float"); } )
          | (ATTR_FLOAT ) ->
;

simpleText    : alwaysText
            | txt=(HTML_BASE | COMMENT | QUOTED | SYMBOLS | LT | GT | CSHARP_BASE | CSHARP_FLOW | CSHARP_SPECIAL | CSHARP_TYPES)
               { Append($txt); }
            | whs=WS       { Append(" "); }
;
pureText : alwaysText
            | txt=(HTML_BASE | COMMENT | QUOTED | SYMBOLS | LT | GT | CSHARP_BASE | CSHARP_FLOW | CSHARP_SPECIAL | CSHARP_TYPES)
               { Append($txt); }
            | whs=WS
;
codeText    : alwaysText
            |  hb=HTML_BASE     { SpanClassTag($hb.Text , "html");}
            | cmt=COMMENT       { SpanClassTag($cmt.Text, "cmmnt");}
            | qtd=QUOTED        { SpanClassTag($qtd.Text, "qtd");  }
            |   s=(SYMBOLS | LT | GT) { SpanClassTag($s.Text, "smbl"); }
            | csb=CSHARP_BASE   { SpanClassTag($csb.Text, "base"); }
            | csf=CSHARP_FLOW   { SpanClassTag($csf.Text, "flw"); }
            | css=CSHARP_SPECIAL{ SpanClassTag($css.Text, "spcl"); }
            | cst=CSHARP_TYPES  { SpanClassTag($cst.Text, "type"); }
            | whs=WS       { Append(WhiteSpaceText($whs.Text)); }
;

alwaysText   :  txt=( WORD | NOCONTROL | ENCODED | COLORS | STYLES) { Append($txt); }
             | esc=ESCAPE   { Append(EscapedText($esc.Text)); }
             | NEWLINE ->
;
